import 'dart:ui';

import 'package:dotup_flutter_prototyping/src/prototype_controller.dart';
import 'package:dotup_flutter_simulator/dotup_flutter_simulator.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import 'prototype_descriptor.dart';
import 'prototype_presenter.dart';
import 'prototype_splash_screen.dart';

class DesktopScrollBehavior extends MaterialScrollBehavior {
  @override
  Set<PointerDeviceKind> get dragDevices => {
        PointerDeviceKind.touch,
        PointerDeviceKind.mouse,
      };
}

class PrototypeApp extends StatefulWidget {
  PrototypeApp({
    Key? key,
    // required this.settingsController,
    required this.prototypeDescriptor,
    this.simulatorController,
    PrototypeController? controller,
  })  : this.controller = controller ??
            PrototypeController(
              descriptor: prototypeDescriptor,
              simulatorController: simulatorController,
            ),
        super(key: key);

  // final SettingsController settingsController;

  final PrototypeDescriptor prototypeDescriptor;

  final SimulatorController? simulatorController;

  final PrototypeController controller;

  static final messenger = GlobalKey<ScaffoldMessengerState>();

  @override
  State<PrototypeApp> createState() => _PrototypeAppState();
}

class _PrototypeAppState extends State<PrototypeApp> {
  late ThemeMode themeMode;

  @override
  void initState() {
    // TODO: implement initState
    final brightness = widget.simulatorController?.brightness == null ? Brightness.dark : widget.simulatorController!.brightness;
    themeMode = brightness == Brightness.dark ? ThemeMode.dark : ThemeMode.light;

    widget.simulatorController?.addListener(checkThemeMode);
  }

  @override
  void dispose() {
    widget.simulatorController?.removeListener(checkThemeMode);
    super.dispose();
  }

  void checkThemeMode() {
    if (widget.simulatorController!.brightness == Brightness.dark && themeMode != ThemeMode.dark) {
      setState(() {
        themeMode = ThemeMode.dark;
      });
    } else if (widget.simulatorController!.brightness == Brightness.light && themeMode != ThemeMode.light) {
      setState(() {
        themeMode = ThemeMode.light;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    // final controller = MockupController(pages: mocks);

    return MaterialApp(
      scaffoldMessengerKey: PrototypeApp.messenger,
      useInheritedMediaQuery: true,
      scrollBehavior: DesktopScrollBehavior(),
      debugShowCheckedModeBanner: false,
      // Providing a restorationScopeId allows the Navigator built by the
      // MaterialApp to restore the navigation stack when a user leaves and
      // returns to the app after it has been killed while running in the
      // background.
      restorationScopeId: 'app',

      theme: ThemeData(),
      darkTheme: ThemeData.dark().copyWith(primaryColor: PROTOTYPE_COLOR, primaryColorDark: PROTOTYPE_COLOR),
      themeMode: themeMode,

      onGenerateRoute: (RouteSettings routeSettings) {
        // final ft = FadeTransition(opacity: 1);

        switch (routeSettings.name) {
          case '/':
            return MaterialPageRoute<void>(
              settings: routeSettings,
              builder: (BuildContext context) {
                return PrototypeSplashScreen(
                  onTap: (showPageInfos) {
                    widget.controller.splashScreenVissible = false;
                    Navigator.of(context).restorablePushReplacementNamed('mockup', arguments: showPageInfos);
                    Future.delayed(Duration(milliseconds: 200)).then((value) => widget.simulatorController?.updateOverlay());
                  },
                );
              },
            );

          // case SettingsView.routeName:
          //   return MaterialPageRoute<void>(
          //     settings: routeSettings,
          //     builder: (BuildContext context) {
          //       return SettingsView(controller: settingsController);
          //     },
          //   );

          default:
            return PageRouteBuilder(
              pageBuilder: (context, animation, secondaryAnimation) {
                final showPageInfos = (routeSettings.arguments ?? false) as bool;
                final mq = MediaQuery.of(context);
                return MediaQuery(
                  data: mq,
                  child: PrototypePresenter(
                    controller: widget.controller,
                    descriptor: widget.prototypeDescriptor,
                    showPageInfos: showPageInfos,
                    // controller: MockupController(pages: mocks),
                  ),
                );
              },
              transitionsBuilder: (context, animation, secondaryAnimation, child) => FadeTransition(
                opacity: animation,
                child: child,
              ),
              transitionDuration: Duration(milliseconds: 1000),
            );

          // return  FadeInPageRoute<void>(
          //   settings: routeSettings,
          //   builder: (BuildContext context) {
          //     return OperationListView(controller: c);
          //   },
          // );
        }
      },
    );

    //   return AnimatedBuilder(
    //     animation: settingsController,
    //     builder: (BuildContext context, Widget? child) {
    //       return MaterialApp(
    //         scaffoldMessengerKey: messenger,
    //         scrollBehavior: DesktopScrollBehavior(),
    //         debugShowCheckedModeBanner: false,
    //         // Providing a restorationScopeId allows the Navigator built by the
    //         // MaterialApp to restore the navigation stack when a user leaves and
    //         // returns to the app after it has been killed while running in the
    //         // background.
    //         restorationScopeId: 'app',

    //         theme: ThemeData(),
    //         darkTheme: ThemeData.dark().copyWith(primaryColor: TOB_COLOR, primaryColorDark: TOB_COLOR),
    //         themeMode: SettingsController.themeMode,

    //         onGenerateRoute: (RouteSettings routeSettings) {
    //           // final ft = FadeTransition(opacity: 1);

    //           switch (routeSettings.name) {
    //             case '/':
    //               return MaterialPageRoute<void>(
    //                 settings: routeSettings,
    //                 builder: (BuildContext context) {
    //                   return MockupSplashScreen(
    //                     onTap: (showPageInfos) => Navigator.of(context).restorablePushReplacementNamed('mockup', arguments: showPageInfos),
    //                   );
    //                 },
    //               );

    //             case SettingsView.routeName:
    //               return MaterialPageRoute<void>(
    //                 settings: routeSettings,
    //                 builder: (BuildContext context) {
    //                   return SettingsView(controller: settingsController);
    //                 },
    //               );

    //             default:
    //               return PageRouteBuilder(
    //                 pageBuilder: (context, animation, secondaryAnimation) {
    //                   final showPageInfos = routeSettings.arguments as bool;
    //                   return MockupPresenter(
    //                     descriptor: mockupDescriptor,
    //                     showPageInfos: showPageInfos,
    //                     // controller: MockupController(pages: mocks),
    //                   );
    //                 },
    //                 transitionsBuilder: (context, animation, secondaryAnimation, child) => FadeTransition(
    //                   opacity: animation,
    //                   child: child,
    //                 ),
    //                 transitionDuration: Duration(milliseconds: 1000),
    //               );

    //             // return  FadeInPageRoute<void>(
    //             //   settings: routeSettings,
    //             //   builder: (BuildContext context) {
    //             //     return OperationListView(controller: c);
    //             //   },
    //             // );
    //           }
    //         },
    //       );
    //     },
    //   );
  }
}

class FadeInPageRoute<T> extends MaterialPageRoute<T> {
  FadeInPageRoute({
    RouteSettings? settings,
    required WidgetBuilder builder,
  }) : super(settings: settings, builder: builder);

  @override
  // TODO: implement barrierColor
  Color get barrierColor => Colors.black;

  @override
  String get barrierLabel => '';

  @override
  Widget buildPage(BuildContext context, Animation<double> animation, Animation<double> secondaryAnimation) {
    return FadeTransition(
      opacity: animation,
      child: builder(context),
    );
  }

  @override
  bool get maintainState => true;

  @override
  Duration get transitionDuration => Duration(milliseconds: 1500);
}
